#define MAINARGS_DEFS
#define GFX_DEFS
#define AUDIO_DEFS
#define MSG_DEFS
#define SDL_DEFS
#define GL_DEFS
#define SYS_DEFS
#define LOOPS_ENTRYPOINT_DEFS
#define CONFIG_DEFS
#define INPUT_DEFS
#define GAME_DEFS
#include "header.h"

#define WORLDW 1536
#define WORLDH 128

extern const char *map1ptr;
extern const char *map2ptr;

namespace Player
{
	bool Subspace();
	bool Alive();
}

namespace Game
{
	unsigned char worldarr[WORLDW][WORLDH]{};
	unsigned char subworldarr[WORLDW][WORLDH]{};
    //                     0   1   2   3   4   5   6   7   8   9  10  11  12  13  14  15  16  17  18  19  20  21  22  23  24  25  26  27  28  29  30  31  32  33
	const char tiles[] = {' ','#','*','/','=','-','q','Q','@','P','p','^','+','|','|',':','[',']','{','}','&','%','$','l','O','S','!','W','H','Y','~','0','_','.','\0'};
	bool solid[256]    = { 0 , 1 , 1 , 0 , 1 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 1 , 0 , 0 , 0 , 0 , 0 , 1 , 0 , 1 , 1 , 0 , 0 , 1 , 0 , 1 , 1 , 0 , 0 , 0};
	bool border1[256]  = { 0 , 1 , 1 , 0 , 1 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 1 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 1 , 1 , 0 , 0 , 0};
	bool border2[256]  = { 0 , 1 , 1 , 1 , 1 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 1 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 1 , 0 , 0 , 0 , 0 , 1 , 1 , 0 , 0 , 0};
	bool back[256]     = { 1 , 1 , 1 , 0 , 1 , 1 , 1 , 1 , 1 , 1 , 1 , 1 , 1 , 1 , 1 , 1 , 1 , 1 , 1 , 1 , 1 , 1 , 0 , 1 , 1 , 1 , 1 , 1 , 1 , 1 , 1 , 1 , 1 , 0};
	int  steptype[256] = { 0 , 1 , 2 , 0 , 3 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 3 , 0 , 0 , 3 , 0 , 0 , 3 , 0 , 2 , 0 , 0 , 0 , 0};
	int steptype2[256] = { 0 , 3 , 3 , 0 , 3 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 3 , 0 , 0 , 3 , 0 , 0 , 3 , 0 , 3 , 0 , 0 , 0 , 0};

	void Init()
	{
		const char *map = map1ptr;
		int x = 0, y = 0;
		while (*map)
		{
			if (*map == 'v')
			{
				x = 0;
				y++;
			}
			else
			{
				int i = 0;
				while (1)
				{
					if (tiles[i] == 0) Sys::ErrExit("Invalid tile in map.");
					if (tiles[i] == *map) break;
					i++;
				}
				worldarr[x][y] = i;
				x++;
			}
			map++;
		}

		map = map2ptr;
		x = y = 0;
		while (*map)
		{
			if (*map == 'v')
			{
				x = 0;
				y++;
			}
			else
			{
				int i = 0;
				while (1)
				{
					if (tiles[i] == 0) Sys::ErrExit("Invalid tile in map.");
					if (tiles[i] == *map) break;
					i++;
				}
				subworldarr[x][y] = i;
				x++;
			}
			map++;
		}
	}

	bool IsSolid(unsigned char id)
	{
		return solid[id];
	}

	int StepSoundOf(int id)
	{
		if (Player::Subspace())
			return steptype2[id];
		else
			return steptype[id];
	}

	int BlockAt(int x, int y)
	{
		if (x < 0 || y < 0 || x > WORLDW*16 || y > WORLDH*16) return 0;
		x = x/16;
		y = y/16;
		if (Player::Subspace())
			return subworldarr[x][y];
		else
			return worldarr[x][y];
	}

	void SetBlockAt(int x, int y, int id)
	{
		if (x < 0 || y < 0 || x > WORLDW*16 || y > WORLDH*16) return;
		x = x/16;
		y = y/16;
		if (Player::Subspace())
			subworldarr[x][y] = id;
		else
			worldarr[x][y] = id;
	}

	void DrawSky()
	{
		if (Player::Subspace())
		{
			GFX::Push(0, 0, 768, 512, 776, 512, 240, 512);
			GFX::Push(220, 130, 48, 48, 96, 376, 48, 48);
			GFX::Push(400, 90, 48, 48, 144, 376, 48, 48);
		}
		else
		{
			GFX::Push(0, 0, 768, 512, 0, 512, 768, 512);
			GFX::Push(550, 100, 48, 48, 0, 48, 48, 48);
		}
	}

	static void S_DrawBlocksBack(int x, int y, int xm, int ym)
	{
		static int frame = 0;
		frame++;
		frame %= 64;

		int xtex = Player::Subspace() ? 256 : 0;
		int ytex = Player::Subspace() ? 424 : 16;

		for (int i = 0; i <= 48; i++)
		{
			for (int j = 0; j <= 32; j++)
			{
				int xx = x + i*16;
				int yy = y + j*16;
				int b = BlockAt(x + i*16, y + j*16);
				if (b && back[b])
				{
					if (b == 20) // rescue pod
					{
						GFX::Push(-xm + i*16 - 16, -ym + j*16 - 32, 48, 48, 0, 96+48*(frame/16), 48, 48);
						continue;
					}
					GFX::Push(-xm + i*16, -ym + j*16, 16, 16, b*16 - 16, ytex, 16, 16);

					if (border1[b])
					{
						GFX::Push(-xm + i*16, -ym + j*16, 16, 16, xtex+16*(!border2[BlockAt(xx, yy-16)] + !border2[BlockAt(xx, yy+16)]*2 + !border2[BlockAt(xx-16, yy)]*4 + !border2[BlockAt(xx+16, yy)]*8), 32, 16, 16);
					}
				}
			}
		}
	}

	static void S_DrawBlocksTop(int x, int y, int xm, int ym)
	{
		int xtex = Player::Subspace() ? 256 : 0;
		int ytex = Player::Subspace() ? 424 : 16;

		for (int i = 0; i <= 48; i++)
		{
			for (int j = 0; j <= 32; j++)
			{
				int xx = x + i*16;
				int yy = y + j*16;
				int b = BlockAt(x + i*16, y + j*16);
				if (b && !back[b])
				{
					GFX::Push(-xm + i*16, -ym + j*16, 16, 16, b*16 - 16, ytex, 16, 16);

					if (border1[b])
					{
						GFX::Push(-xm + i*16, -ym + j*16, 16, 16, xtex+16*(!border2[BlockAt(xx, yy-16)] + !border2[BlockAt(xx, yy+16)]*2 + !border2[BlockAt(xx-16, yy)]*4 + !border2[BlockAt(xx+16, yy)]*8), 32, 16, 16);
					}
				}
			}
		}
	}

	int drawx = 768/2-16, drawy = 512/2-16+64;
	void DrawChar(int x, int y, int state) // standL, standR, runL, runR, jumpL, jumpR, fallL, fallR, deadL, deadR
	{
		x -= drawx;
		y -= drawy;

		int xm = x % 16, ym = y % 16;
		if (xm < 0) xm += 16;
		if (ym < 0) ym += 16;

		S_DrawBlocksBack(x, y, xm, ym);

		static int frame = 0;
		static constexpr int fr = 6;
		if ((state >= 8 && state < 12) || (state == 14 || state == 15))
			frame = 0;
		else
		{
			frame++;
			frame %= 8*fr;
		}

		GFX::Push(drawx, drawy, 32, 32, 48 + state * 32, 48 + frame / fr * 32, 32, 32);

		S_DrawBlocksTop(x, y, xm, ym);
	}

	void DrawBlocksOnly(int x, int y)
	{
		x -= drawx;
		y -= drawy;

		int xm = x % 16, ym = y % 16;
		if (xm < 0) xm += 16;
		if (ym < 0) ym += 16;
		S_DrawBlocksBack(x, y, xm, ym);
		S_DrawBlocksTop(x, y, xm, ym);
	}

	void DrawText(const char *txt, int x, int y, int align, const GFX::GammaData &g)
	{
        static const char * const font = " !\"#$%&'()*+,-./0123456789:;<=>?@ABCDEFGHIJKLMNOPQRSTUVWXYZ[\\]^_`abcdefghijklmnopqrstuvwxyz{|}~";
        const char *fp;
        if (align == 1)
			x -= std::strlen(txt) * 6;
		else if (align == 0)
			x -= std::strlen(txt) * 3;
        while (*txt)
		{
			fp = font;
			while (*fp)
			{
				if (*fp == *txt)
				{
					GFX::PushGamma(x, y, 6, 10, 64 + 6 * (fp - font), 0, 6, 10, g);
					x += 6;
					break;
				}
				fp++;
			}
			txt++;
		}
	}

	void DrawText(std::initializer_list<const char *> list, int x, int y, int align, const GFX::GammaData &g)
	{
		for (auto it : list)
		{
			DrawText(it, x, y, align, g);
			y += 10;
		}
	}

}