#include <sstream>
#include <fstream>
#include <iostream>
#include <string>
#include <initializer_list>
#include <random>
#include <type_traits>
#include <cstring>
#include <cstdint>
#include <cmath>
#include <cstdio>
#include <csignal>
#include <ctime>
#include <cstddef>


#define APPNAME                    "TheThing"
#define TICKLEN                    10
#define ATEXITSTACKSIZE            32
#define AUDIOCHANNELS              64
#define HQAUDIO                    1
#define AUDIOHALFDISTBETWEENEARS   0.5
#define AUDIOPOSMULTIPLICATOR      1
#define IMAGEBUFFERSIZE            65536
#define TEXSIZE                    1024
#define TEXSIZE_STRING             "1024"
#define SDL_HEADER_PATH            <SDL2/SDL.h>
#define SDL_IMAGE_HEADER_PATH      <SDL2/SDL_image.h>
#define SDL_MIXER_HEADER_PATH      <SDL2/SDL_mixer.h>
#define GLEW_HEADER_PATH           <GL/glew.h>

#define GLERRCHECK() if (glGetError()) Msg::MsgStream << Msg::Clear << "Error at line " << __LINE__ << Sys::ErrExit

/**
 * SDL_DEFS
 * GL_DEFS
 * MAINARGS_DEFS
 * AUDIO_DEFS
 * GFX_DEFS
 * SYS_DEFS
 * LOOPS_ENTRYPOINT_DEFS
 * MSG_DEFS
 * INPUT_DEFS
 * CONFIG_DEFS
 * GAME_DEFS
 *
 */



/// SDL_DEFS
#ifdef SDL_DEFS
#undef SDL_DEFS
#include SDL_HEADER_PATH
#include SDL_IMAGE_HEADER_PATH
#include SDL_MIXER_HEADER_PATH
#endif

/// GL_DEFS
#ifdef GL_DEFS
#undef GL_DEFS
#include GLEW_HEADER_PATH
#endif

/// MAINARGS_DEFS
#ifdef MAINARGS_DEFS
#undef MAINARGS_DEFS
namespace MainArgs
{
	const char *ExecutableName();
	unsigned int ArgCount();
	const char *ArgAt(unsigned int);
	bool ArgExist(const char *);
	bool ArgWithPrefixExist(const char *);
	unsigned int LastArg(); // ArgExist() and ArgWithPrefixExist()
}
#endif

/// AUDIO_DEFS
#ifdef AUDIO_DEFS
#undef AUDIO_DEFS
namespace Audio
{
	namespace Music
	{
		using Type = Mix_Music*;

		Type Load(const char *); // .mp3 & .ogg only
		void Set(Mix_Music *); // param can be 0
		void Pause(bool);
	}

	namespace Sound
	{
		using Type = Mix_Chunk*;

		Type Load(const char *); // .wav only
		void Play(Type sound);
		void Play(Type sound, float x, float y); // Play(a) != Play(a, 0, 0)
	}
}
#endif

/// GFX_DEFS
#ifdef GFX_DEFS
#undef GFX_DEFS
namespace GFX
{
	extern const double Pi_d;
	extern const float Pi_f;
	struct GammaData
	{
		float g[8];
	};
	GammaData Gamma(short rmin, short rmax, short gmin, short gmax, short bmin, short bmax, short amin, short amax); // rmin rmax gmin gmax bmin bmax amin amax

	// Separate gamma: g1 - top-left, g2 - top-right, g3 - bottom-left, g4 - bottom-right

	void Push(float x, float y, float w, float h, float tx, float ty, float tw, float th);
	void PushExt(float x, float y, float w, float h, float tx, float ty, float tw, float th, float angle, float rx, float ry);
	void PushGamma(float x, float y, float w, float h, float tx, float ty, float tw, float th, GammaData g);
	void PushSeparateGamma(float x, float y, float w, float h, float tx, float ty, float tw, float th, GammaData g1, GammaData g2, GammaData g3, GammaData g4);
	void PushExtGamma(float x, float y, float w, float h, float tx, float ty, float tw, float th, float angle, float rx, float ry, GammaData g);
	void PushExtSeparateGamma(float x, float y, float w, float h, float tx, float ty, float tw, float th, float angle, float rx, float ry,
							  GammaData g1, GammaData g2, GammaData g3, GammaData g4);

	void PosPush(unsigned int pos, float x, float y, float w, float h, float tx, float ty, float tw, float th);
	void PosPushExt(unsigned int pos, float x, float y, float w, float h, float tx, float ty, float tw, float th, float angle, float rx, float ry);
	void PosPushGamma(unsigned int pos, float x, float y, float w, float h, float tx, float ty, float tw, float th, GammaData g);
	void PosPushSeparateGamma(unsigned int pos, float x, float y, float w, float h, float tx, float ty, float tw, float th,
							  GammaData g1, GammaData g2, GammaData g3, GammaData g4);
	void PosPushExtGamma(unsigned int pos, float x, float y, float w, float h, float tx, float ty, float tw, float th, float angle, float rx, float ry,
						 GammaData g);
	void PosPushExtSeparateGamma(unsigned int pos, float x, float y, float w, float h, float tx, float ty, float tw, float th, float angle, float rx, float ry,
								 GammaData g1, GammaData g2, GammaData g3, GammaData g4);

	short WinXX();
	short WinYY();

	extern const GammaData stdgamma;
}
#endif

/// GAME_DEFS
#ifdef GAME_DEFS
#undef GAME_DEFS
namespace Game
{
	void Init();
	int BlockAt(int x, int y);
	void SetBlockAt(int x, int y, int id);
	bool IsSolid(unsigned char id);
	int StepSoundOf(int id);
	void DrawSky();
	void DrawChar(int x, int y, int state);
	void DrawBlocksOnly(int x, int y);
	void DrawText(const char *, int x, int y, int align, const GFX::GammaData & = GFX::stdgamma);
	void DrawText(std::initializer_list<const char *>, int x, int y, int align, const GFX::GammaData & = GFX::stdgamma);
}
#endif

/// SYS_DEFS
#ifdef SYS_DEFS
#undef SYS_DEFS
namespace Sys
{
	void Exit();
	void ErrExit(const char *);
	void AtExit(void (*)());
	unsigned int Delta();
	void Tick();
}
#endif

/// CONFIG_DEFS
#ifdef CONFIG_DEFS
#undef CONFIG_DEFS
namespace Config
{
	namespace Volume
	{
		void Music(float val);
		void Sound(float val);
	}
}
#endif

/// LOOPS_ENTRYPOINT_DEFS
#ifdef LOOPS_ENTRYPOINT_DEFS
#undef LOOPS_ENTRYPOINT_DEFS
namespace Loops
{
	void EntryPoint();
}
#endif

/// INPUT_DEFS
#ifdef INPUT_DEFS
#undef INPUT_DEFS
namespace Input
{
	/**
	 * Really pressed? 0 0 1 1 1 0 0
	 * Up()            1 1 0 0 0 0 1
     * Down()          0 0 1 1 1 0 0
     * Hold()          0 0 0 1 1 0 0
     * Pressed()       0 0 1 0 0 0 0
     * Released()      0 0 0 0 0 1 0
     *               ---------------->
	 */
	namespace Mouse
	{
		void Show(bool);
		void Grab(bool);

		int PosX();
		int PosY();
		int AbsPosX();
		int AbsPosY();

		bool Inside();

		bool Up(unsigned char);       // param >= 1
		bool Down(unsigned char);     //
		bool Hold(unsigned char);     // LMB  MMB  RMB  ...
		bool Pressed(unsigned char);  //
		bool Released(unsigned char); //  1    2    3   ...

		bool WheelUp();
		bool WheelDown();
		bool WheelLeft();
		bool WheelRight();

		unsigned char PressedNow();

		bool RectHover(int x, int y, int w, int h);
		bool RectClick(int x, int y, int w, int h);
		bool RectHold(int x, int y, int w, int h);
	}

	namespace Keyboard
	{
		unsigned int NowPressed();
		bool Up(unsigned int);
		bool Down(unsigned int);
		bool Hold(unsigned int);
		bool Pressed(unsigned int);
		bool Released(unsigned int);
		bool Repeated(unsigned int);
		bool PressedOrRepeated(unsigned int);
	}
}
#endif

/// MSG_DEFS
#ifdef MSG_DEFS
#undef MSG_DEFS
namespace Msg
{
	void Info(const char *txt);
	void Warn(const char *txt);
	void Clear(std::stringstream &ss);

	class MsgStreamClass
	{
		static std::stringstream ss;

		MsgStreamClass();
		MsgStreamClass(const MsgStreamClass &);
		MsgStreamClass(MsgStreamClass &&);
		MsgStreamClass &operator=(const MsgStreamClass &);
		MsgStreamClass &operator=(MsgStreamClass &&);

	public:
		MsgStreamClass &operator<<(void (*ptr)(const std::string &))
		{
			ptr(ss.str());
			return *this;
		}

		MsgStreamClass &operator<<(void (*ptr)(const char *))
		{
			ptr(ss.str().c_str());
			return *this;
		}

		MsgStreamClass &operator<<(void (*ptr)(std::stringstream &))
		{
			ptr(ss);
			return *this;
		}

		static MsgStreamClass &Inst();

		template <typename T1> MsgStreamClass &operator<<(const T1 &obj)
		{
			ss << obj;
			return *this;
		}
	};

	/** Usage:
	 *   Execute a function:  MsgStream << (*)(const std::string &);
	 *                        MsgStream << (*)(const char *);
	 *                        MsgStream << (*)(std::stringstream &);
	 *   Add obj to string:   MsgStream << somesthing;
	 *
	 *   Example:
	 *    Msg::MsgStream << Msg::Clear << "Test!" << Msg::Info;
	 */
	extern MsgStreamClass &MsgStream;
}
#endif


namespace Inline
{
	inline void StopExecution()
	{
		volatile unsigned int var = 0;
		while (1) var++;
	}

	inline std::clock_t Time()
	{
		return std::clock() * 1000 / CLOCKS_PER_SEC;
	}
}
