#define LOOPS_ENTRYPOINT_DEFS
#define SYS_DEFS
#define MSG_DEFS
#define INPUT_DEFS
#define GL_DEFS
#define CONFIG_DEFS
#define SDL_DEFS
#define GFX_DEFS
#define AUDIO_DEFS
#define GAME_DEFS
#include "header.h"

//#define DEBUG_MOVEMENT

namespace Game
{
	extern int drawx, drawy;
}

namespace GameAudio
{
	Audio::Music::Type back;
	Audio::Sound::Type door;
	Audio::Sound::Type step[3];
	Audio::Sound::Type podlanding;
	Audio::Sound::Type shift1;
	Audio::Sound::Type shift2;
	Audio::Sound::Type cpoint;
	Audio::Sound::Type pick;
	Audio::Sound::Type blip;
	Audio::Sound::Type burn;
	Audio::Sound::Type br;
	Audio::Sound::Type roar;
	Audio::Sound::Type roar2;
	Audio::Sound::Type tp1;
	Audio::Sound::Type tp2;
}

namespace Player
{
	static int x = 2164, y = 350;
	static float xv = 0, yv = -0.3;
	static bool alive = 1;
	static int speech = 0, sptimer = 0;
	static bool havering = 0, subspace = 0;
	static int ringtimer = 1920, deathtimer = 500;
	static int cpoint = 0;

	static float a_x = 1016, a_y = 448;
	static bool a_angry = 0;
	static int a_timer = -150;

	namespace Save
	{
		static int x = 2164, y = 350;
		static bool havering = 0, subspace = 0;
		static int ringtimer = 1920;

		static float a_x = 1016, a_y = 448;
		static bool a_angry = 0;
		static int a_timer = -150;
	}

	static void LoadSave()
	{
		x = Save::x;
		y = Save::y;
		xv = yv = 0;
		alive = 1;
		havering = Save::havering;
		subspace = Save::subspace;
		ringtimer = Save::ringtimer;
		deathtimer = 500;

		a_x = Save::a_x;
		a_y = Save::a_y;
		a_angry = Save::a_angry;
		a_timer = Save::a_timer;
	}

	static void CreateSave()
	{
		Save::x = x;
		Save::y = y;
		Save::havering = havering;
		Save::subspace = subspace;
		Save::ringtimer = ringtimer = 1920;

		Save::a_x = a_x;
		Save::a_y = a_y;
		Save::a_angry = a_angry;
		Save::a_timer = a_timer;
	}

	static void Say(int id)
	{
		speech = id;
		sptimer = 200;
	}

	constexpr int cplist_x[] = {2889,4153,5847,9999};
	constexpr int cplist_y[] = { 240, 736, 432,-100};
	static void CheckCP()
	{
		static int phase = 0;
		phase++;
		phase %= 192;
		if ((cpoint == 2) != subspace) return;
		if (cpoint == 1 && y < 735) return;
		int ph = phase/24;
		int cx = cplist_x[cpoint]-x+16;
		int cy = cplist_y[cpoint]-y+16;
		if (ph < 3)
			GFX::Push(cx+Game::drawx-8, cy+Game::drawy-16-1, 16, 32, 48, 344, 16, 32);
		else if (ph == 3 || ph == 7)
			GFX::Push(cx+Game::drawx-8, cy+Game::drawy-16, 16, 32, 48, 344, 16, 32);
		else
			GFX::Push(cx+Game::drawx-8, cy+Game::drawy-16+1, 16, 32, 48, 344, 16, 32);

		if (cx < 16)
		{
			CreateSave();
			cpoint++;
			Audio::Sound::Play(GameAudio::cpoint);
			Say(0);
		}
	}

	bool Subspace()
	{
		return subspace;
	}

	bool Alive()
	{
		return alive;
	}

	static bool IsSolidAt(int xoff, int yoff)
	{
		static const auto P = [](int xo, int yo) -> bool
		{
			return Game::IsSolid(Game::BlockAt(xo, yo));
		};

		return P(x+xoff+13, y+yoff+3) || P(x+xoff+18, y+yoff+3) || P(x+xoff+13, y+yoff+17) ||
			   P(x+xoff+18, y+yoff+17) || P(x+xoff+13, y+yoff+31) || P(x+xoff+18, y+yoff+31);
	}

	static bool Tick()
	{
		if (havering)
		{
			if (!a_angry)
			{
				a_timer++;
				if (a_timer > 200)
					a_angry = 1;
				else if (a_timer == -20)
					Audio::Sound::Play(GameAudio::roar);
			}
			else
			{
				if (a_timer > 0)
				{
					if (a_timer == 1)
					{
						Audio::Sound::Play(GameAudio::roar2);
						Say(3);
					}
					a_timer--;
				}
			}
		}
		if (a_angry && a_timer == 0)
		{
			float vx = x-a_x;
			float vy = y-a_y;

			if (vx < 0) vx = -vx;
			if (vy < 0) vy = -vy;

			static constexpr float v1 = 0.05, f = 100.0;
			float dist = std::sqrt(vx*vx + vy*vy);
			if (dist > 16)
			{
				if (vx > 3)
				{
					if (x > a_x) a_x += vx * v1 / dist + dist / f;
					else            a_x -= vx * v1 / dist + dist / f;
				}

				if (vy > 3)
				{
					if (y > a_y)  a_y += vy * v1 / dist + dist / f;
					else            a_y -= vy * v1 / dist + dist / f;
				}
			}
			else
			{
				alive = 0;
			}
		}

		//std::cout << '\t' << x << '\t' << y << '\n';
		#ifdef DEBUG_MOVEMENT
		if (Input::Keyboard::Down(4 )) x -= 32;
		if (Input::Keyboard::Down(7 )) x += 32;
		if (Input::Keyboard::Down(26)) y -= 32;
		if (Input::Keyboard::Down(22)) y += 32;
		#endif // DEBUG_MOVEMENT

		if (xv < 0 && IsSolidAt(-1, 0)) xv = 0;
		if (yv < 0 && IsSolidAt(0, -1)) yv = 0;
		if (xv > 0 && IsSolidAt(+1, 0)) xv = 0;
		if (yv > 0 && IsSolidAt(0, +1)) yv = 0;

		if (y < 0)
		{
			y = 0;
			if (yv < 0)
				yv = 0;
		}

		int tmpv = xv * 10;
		if (tmpv)
		{
			if (xv > 0)
			{
				for (int i = 0; i < tmpv; i++)
				{
					if (!IsSolidAt(1, 0))
						x++;
				}
			}
			else
			{
				for (int i = 0; i > tmpv; i--)
				{
					if (!IsSolidAt(-1, 0))
						x--;
				}
			}
		}

		tmpv = yv * 10;
		if (tmpv)
		{
			if (yv > 0)
			{
				for (int i = 0; i < tmpv; i++)
				{
					if (!IsSolidAt(0, 1))
						y++;
				}
			}
			else
			{
				for (int i = 0; i > tmpv; i--)
				{
					if (!IsSolidAt(0, -1))
						y--;
				}
			}
		}

		bool moved = 0;

		const bool onground = IsSolidAt(0, 1);

		if (Input::Keyboard::Down(80) && alive && x < 7000)
		{
			xv -= 10 * 0.003;
			moved = 1;
		}
		if ((Input::Keyboard::Down(79) && alive) || x >= 7000)
		{
			xv += 10 * 0.003;
			moved = 1;
		}

		static bool doublejump = 0;

		if (onground)
		{
			if (!IsSolidAt(0, 0))
				doublejump = 1;

			if (Game::BlockAt(x + 13, y + 32) == 30 ? (Game::SetBlockAt(x + 13, y + 33, 25),1) : 0
			 || Game::BlockAt(x + 18, y + 32) == 30 ? (Game::SetBlockAt(x + 18, y + 33, 25),1) : 0)
			{
				Audio::Sound::Play(GameAudio::br, 0, 2);
			}

			if (moved)
			{
				static int walksoundst = 0;
				static constexpr int steplen = 16;
				walksoundst += 1;
				walksoundst %= steplen;

				int st = Game::StepSoundOf(Game::BlockAt(x + 16, y + 33));

				if (!walksoundst)
				{
					if (st)
					{
						Audio::Sound::Play(GameAudio::step[st-1], 0, 4);
					}
				}
			}
		}

		if (Input::Keyboard::Pressed(82) && alive && (onground || doublejump) && x < 7000)
		{
			if (!onground)
				doublejump = 0;
			yv = -0.45;
		}

		if (subspace)
			yv += 6 * 0.0015;
		else
			yv += 10 * 0.0015;

		if (!moved)
			for (unsigned int i = 0; i < 10; i++)
				xv /= 1.1;



		static constexpr float xvlimit = 0.2, yvlimit = 1;
		if (xv > xvlimit) xv = xvlimit;
		if (yv > yvlimit) yv = yvlimit;
		if (xv < -xvlimit) xv = -xvlimit;
		if (yv < -yvlimit) yv = -yvlimit;


		static constexpr float anim_v = 0.1;

		int state;
		if (alive)
		{
			if (onground)
			{
				if ((int)(xv*100) == 0)
					state = 0;
				else
					state = 2;
			}
			else
			{
				if (yv < 0)
					if (yv < -anim_v)
						state = 4;
					else
						state = 8;
				else
					if (yv > anim_v)
						state = 6;
					else
						state = 10;
			}
		}
		else
		{
			if (onground)
				if (subspace)
					state = 16;
				else
					state = 12;
			else
				state = 14;

			if (deathtimer > 0) deathtimer--;
		}
		static bool dir = 0;

		if (xv > 0)
			dir = 1;
		if (xv < 0)
			dir = 0;
		if (dir)
			state += 1;

		Game::DrawChar(x, y, state);
		if (subspace && alive)
		{
			static GFX::GammaData g1;
			g1 = GFX::Gamma(0, 255, 0, 255, 0, 255, 0, ringtimer * 255.0 / 1920.0);
			GFX::PushGamma(768/2-24, 512/2-24+64, 48, 48, 0, 456, 48, 48, g1);
		}


		if (a_angry) // Alien rendering
		{
			GFX::Push((int)a_x+Game::drawx-x-24+8, (int)a_y+Game::drawy-y-24+16, 48, 48, 192 + 48*(x > a_x) + 96*subspace, 344, 48, 48);
		}
		else
		{
			int dist = x - a_x;
			if (dist < 0) dist = -dist;

			if (dist > 220)
			{
				static int frame = 0;
				frame++;
				frame %= 128;
				if (frame < 64)
					GFX::Push(a_x+Game::drawx-x, a_y+Game::drawy-y, 16, 32, 96 + 64*subspace, 344, 16, 32);
				else
					GFX::Push(a_x+Game::drawx-x, a_y+Game::drawy-y, 16, 32, 112 + 64*subspace, 344, 16, 32);
			}
			else
			{
				if (x < a_x)
					GFX::Push(a_x+Game::drawx-x, a_y+Game::drawy-y, 16, 32, 64 + 64*subspace, 344, 16, 32);
				else
					GFX::Push(a_x+Game::drawx-x, a_y+Game::drawy-y, 16, 32, 80 + 64*subspace, 344, 16, 32);
			}
		}
		if (a_timer > 0)
		{
			static GFX::GammaData g;
			g = GFX::Gamma(0, 255*subspace, 0, 255*subspace, 0, 255*subspace, 0, a_timer*5);
			GFX::PushGamma(a_x+Game::drawx-x + 8 - 47, a_y+Game::drawy-y + 16 - 47, 94, 94, 49, 457, 46, 46, g);
		}


		if (havering)
		{
			if (!subspace)
			{
				GFX::Push(768/2-48/2, 512-48, 48, 48, 0, 376, 48, 48);
				if (!IsSolidAt(0, 0))
					ringtimer += 1;
			}
			else
			{
				GFX::Push(768/2-48/2, 512-48, 48, 48, 48, 376, 48, 48);
				ringtimer -= 1;

				if (ringtimer % 100 == 0 && ringtimer <= 500 && alive)
				{
					Audio::Sound::Play(GameAudio::blip, 0, 1);
				}
			}
			if (ringtimer > 1920) ringtimer = 1920;
			if (ringtimer < 0)
			{
				ringtimer = 0;
				alive = 0;
			}
			if (ringtimer != 1920)
			{
				GFX::Push(769/2 - 192/2, 512-48, 192, 8, 0, 440, 192, 8);
				GFX::Push(769/2 - 192/2, 512-48, ringtimer/10, 8, 0, 448, ringtimer/10, 8);
			}
		}

		bool dooropened = 0;
		if (Game::BlockAt(x + 16, y + 20) == 13 && alive) // door 1
		{
			Game::SetBlockAt(x + 16, y + 20, 14);
			dooropened = 1;
		}
		if (Game::BlockAt(x + 16, y + 12) == 13 && alive) // door 2
		{
			Game::SetBlockAt(x + 16, y + 12, 14);
			dooropened = 1;
		}

		if (dooropened) Audio::Sound::Play(GameAudio::door, 0, 1);

		if (sptimer > 0)
		{
			GFX::Push(768/2, 512/2, 48, 40, speech*48, 304, 48, 40);
			sptimer--;
		}

		// Sound
		static bool al = alive;
		if (al != alive)
		{
			if (!alive && subspace) Audio::Sound::Play(GameAudio::burn);
			al = alive;
		}

		if (Input::Keyboard::Pressed(29) && havering && alive) // Ring
		{
			subspace = !subspace;
			if (subspace) Audio::Sound::Play(GameAudio::shift1);
			else          Audio::Sound::Play(GameAudio::shift2);
		}
		if (Input::Keyboard::Pressed(81) && alive) // Interaction
		{
			int b = Game::BlockAt(x + 16, y + 20);

			if (b == 22) // ring
			{
				Audio::Sound::Play(GameAudio::pick, 0, 2);
				havering = 1;
				Say(2);
				Game::SetBlockAt(x + 16, y + 20, 0);
				CreateSave();
			}
		}

		state/*reuse*/= Game::BlockAt(x+16, y+24); // Block Effects
		if (state == 26 || state == 32) // spikes
		{
			alive = 0;
		}
		else if (state == 33) // stone
		{
			havering = 0;
			ringtimer = 1920;
			subspace = 0;
			return 1;
		}


		if (Input::Keyboard::Pressed(20) && alive) // Suicide
		{
			alive = 0;
		}
		if (!alive && Input::Keyboard::NowPressed() && deathtimer < 450) // Respawn
		{
			LoadSave();
		}

		if (alive)
			CheckCP();

		static const GFX::GammaData g1 = GFX::Gamma(0, 0, 0, 0, 0, 0, 0, 255);
		static const GFX::GammaData g2 = GFX::Gamma(0, 255, 0, 255, 0, 255, 0, 255);
		if (!alive)
		{
			if (subspace)
				Game::DrawText({"YOU DIED", "", "Press any key to load last save"}, 768/2, 512/2, 0, g1);
			else
				Game::DrawText({"YOU DIED", "", "Press any key to load last save"}, 768/2, 512/2, 0, g2);
		}

		return 0;
	}
}

namespace Loops
{
	static void Logo();
	static void Game();

	static void (*nextf)();
	static void NextFunc(void (*ptr)())
	{
		nextf = ptr;
	}
	void EntryPoint()
	{
		static int first = 0;
		if (first++)
			Sys::ErrExit("Entry point func called twice!");

		NextFunc(Logo);

		while (1)
			nextf();
	}

	void Logo()
	{
		GameAudio::back = Audio::Music::Load("audio\\wind.mp3");

		GameAudio::door = Audio::Sound::Load("audio\\door.wav");
		GameAudio::step[0] = Audio::Sound::Load("audio\\dirtstep.wav");
		GameAudio::step[1] = Audio::Sound::Load("audio\\grassstep.wav");
		GameAudio::step[2] = Audio::Sound::Load("audio\\solidstep.wav");
		GameAudio::podlanding = Audio::Sound::Load("audio\\podlanding.wav");
		GameAudio::shift1 = Audio::Sound::Load("audio\\shift.wav");
		GameAudio::shift2 = Audio::Sound::Load("audio\\shiftback.wav");
		GameAudio::cpoint = Audio::Sound::Load("audio\\checkpoint.wav");
		GameAudio::pick = Audio::Sound::Load("audio\\pickring.wav");
		GameAudio::blip = Audio::Sound::Load("audio\\blip.wav");
		GameAudio::burn = Audio::Sound::Load("audio\\burn.wav");
		GameAudio::br = Audio::Sound::Load("audio\\break.wav");
		GameAudio::roar = Audio::Sound::Load("audio\\roar.wav");
		GameAudio::roar2 = Audio::Sound::Load("audio\\roar2.wav");
		GameAudio::tp1 = Audio::Sound::Load("audio\\tp1.wav");
		GameAudio::tp2 = Audio::Sound::Load("audio\\tp2.wav");

		float fadein = 0;
		bool state = 0;
		GFX::GammaData g1, g2, g3;
		g2 = GFX::Gamma(0, 127, 0, 127, 0, 127, 0, 255);
		g3 = GFX::Gamma(0, 64, 0, 64, 0, 64, 0, 255);
		while (1)
		{
			Sys::Tick();
			if (state)
			{
				if (fadein > 0) fadein -= 10 / 4.0;
				else
				{
					NextFunc(Game);
					return;
				}
			}
			else
			{
				if (fadein < 255) fadein += 10 / 4.0;
				else              fadein = 255;
			}
			g2 = GFX::Gamma(0, 127, 0, 127, 0, 127, 0, 255);
			g2 = GFX::Gamma(0, 127, 0, 127, 0, 127, 0, 255);
			Game::DrawSky();
			Game::DrawText({"         Ludum Dare #30 \"Connected Worlds\"",
							"`Dimension jumper`, a game by HolyBlackCat"}, 768, 0, 1, g2);

			GFX::Push(768/2-128, 64, 256, 128, 640, 0, 256, 128);

			Game::DrawText({" Controls:",
							"",
							"Left,Right  - Movement",
							"Up          - Jump (press in midair to doublejump)",
							"Down        - Pick up item",
							"Z           - Use item",
							"Q           - Retry to last checkpoint",
							"Tab         - Restart",
							"Ctrl+Enter  - Toggle graphics mode",
							"Alt+Enter   - Toggle fullscreen"}, 768/2 - 150, 200, -1,g3);

			Game::DrawText("Press any key to start!", 768/2, 384, 0, g2);

			if (Input::Keyboard::NowPressed())
			{
				if (state == 0)
					Audio::Music::Set(GameAudio::back);
				state = 1;
			}

			if (Input::Mouse::Inside())
				GFX::Push(Input::Mouse::PosX(), Input::Mouse::PosY(), 16, 16, 0, 0, 16, 16);

			g1 = GFX::Gamma(0, 255, 0, 255, 0, 255, 255 - fadein, 255 - fadein);
			GFX::PushGamma(0, 0, 768, 512, 0.5, 512.5, 0, 0, g1);
		}
	}

	void Game()
	{
		float fadein = 255;
		GFX::GammaData g1;
		int state = 0, timer = 250;
		while (1) ///
		{
			Sys::Tick();
			if (fadein > 0) fadein -= 10 / 4.0;
			else              fadein = 0;

			Game::DrawSky();
			Game::DrawBlocksOnly(2164, 350);

			if (fadein != 0)
			{
				g1 = GFX::Gamma(0, 255, 0, 255, 0, 255, fadein, fadein);
				GFX::PushGamma(0, 0, 768, 512, 0.5, 512.5, 0, 0, g1);
			}

			if (state == 2)
			{
				Game::SetBlockAt(Player::x+16, Player::y+32, 19);
				Game::SetBlockAt(Player::x, Player::y+32, 18);
				Player::Say(1);
				break;
			}
			else if (state == 1 && timer == 300)
			{
				Audio::Sound::Play(GameAudio::podlanding);
			}

			timer++;
			if (timer > 400)
			{
				state++;
				timer = 0;
			}

			if (state == 1)
			{
				GFX::Push(768 - timer * 4, -5 + timer, 48, 32, 0, 344, 48, 32);
			}
		}


		restart: ;
		while (1)
		{
			Sys::Tick();
			if (Input::Keyboard::Pressed(43)) // Tab
			{
				{
					using namespace Player;
					x = 2164;
					y = 350;
					xv = 0;
					yv = -0.3;
					alive = 1;
					speech = 0;
					sptimer = 0;
					havering = 0;
					subspace = 0;
					ringtimer = 1920;
					deathtimer = 500;
					cpoint = 0;
					a_x = 1016;
					a_y = 448;
					a_angry = 0;
					a_timer = -150;
				}
				{
					using namespace Player::Save;
					x = 2164;
					y = 350;
					havering = 0;
					subspace = 0;
					ringtimer = 0;
					goto restart;
				}
			}

			Game::DrawSky();

			if (Player::Tick())
				break;
		}

		int fintimer = 0;
		while (1)
		{
			Sys::Tick();
			Game::DrawSky();
			Game::DrawBlocksOnly(Player::x, Player::y);

			if (fintimer < 50)
			{
				GFX::Push(Game::drawx+16+fintimer/4, Game::drawy, 32, 32, 312, 88, 32, 32);
				GFX::Push((int)Player::a_x+Game::drawx-Player::x-24+8, (int)Player::a_y+Game::drawy-Player::y-24+16, 48, 48, 192 + 48*(Player::x > Player::a_x), 344, 48, 48);
				GFX::Push(Game::drawx+fintimer/1+32, Game::drawy+16, 16, 16, 336, 16, 16, 16);
			}
			else if (fintimer < 120)
			{
				GFX::Push(Game::drawx+16+50/4, Game::drawy, 32, 32, 312, 88, 32, 32);
				GFX::Push((int)Player::a_x+Game::drawx-Player::x-24+8, (int)Player::a_y+Game::drawy-Player::y-24+16, 48, 48, 192 + 48*(Player::x > Player::a_x), 344, 48, 48);
				GFX::Push(Game::drawx+50/1+32, Game::drawy+16, 16, 16, 336, 16, 16, 16);
			}
			else if (fintimer < 200)
			{
				GFX::Push(Game::drawx+16+50/4, Game::drawy, 32, 32, 48, 48, 32, 32);
				GFX::Push((int)Player::a_x+Game::drawx-Player::x-24+8, (int)Player::a_y+Game::drawy-Player::y-24+16, 48, 48, 192 + 48*(Player::x > Player::a_x), 344, 48, 48);
				GFX::Push(Game::drawx+50/1+32, Game::drawy+16, 16, 16, 336, 16, 16, 16);
			}
			else if (fintimer < 300)
			{
				if (fintimer == 200)
					Audio::Sound::Play(GameAudio::tp1);
				g1 = GFX::Gamma(0, 255, 0, 255, 0, 255, 0, (105 + fintimer % 10 * 10) * (100 - fintimer + 200) / 100.0);
				GFX::Push(Game::drawx+16+50/4, Game::drawy, 32, 32, 48, 48, 32, 32);
				GFX::PushGamma((int)Player::a_x+Game::drawx-Player::x-24+8, (int)Player::a_y+Game::drawy-Player::y-24+16, 48, 48, 192 + 48*(Player::x > Player::a_x), 344, 48, 48, g1);
				GFX::Push(Game::drawx+50/1+32, Game::drawy+16, 16, 16, 336, 16, 16, 16);
			}
			else if (fintimer < 400)
			{
				if (fintimer == 300)
					Audio::Sound::Play(GameAudio::tp2);
				g1 = GFX::Gamma(0, 255, 0, 255, 0, 255, 0, (105 + fintimer % 10 * 10) * (fintimer - 300) / 100.0);
				GFX::Push(Game::drawx+16+50/4, Game::drawy, 32, 32, 80, 48, 32, 32);
				GFX::PushGamma(Game::drawx + 60, Game::drawy - 10, 48, 48, 192, 344, 48, 48, g1);
				GFX::Push(Game::drawx+50/1+32, Game::drawy+16, 16, 16, 336, 16, 16, 16);
			}
			else if (fintimer < 600)
			{
				if (fintimer == 400)
					Audio::Sound::Play(GameAudio::roar);
				GFX::Push(Game::drawx+16+50/4, Game::drawy, 32, 32, 80, 48, 32, 32);
				GFX::Push(Game::drawx + 60, Game::drawy - 10, 48, 48, 192, 344, 48, 48);

				g1 = GFX::Gamma(0, 0, 0, 0, 0, 0, 0, (fintimer-400)*5);

				GFX::Push(Game::drawx+50/1+32, Game::drawy+16, 16, 16, 336, 16, 16, 16);
				GFX::PushGamma(Game::drawx + 60-24, Game::drawy - 10-24, 94, 94, 49, 457, 46, 46, g1);
			}
			else if (fintimer < 800)
			{
				GFX::Push(Game::drawx+16+50/4, Game::drawy, 32, 32, 80, 48, 32, 32);
				GFX::Push(Game::drawx + 60 + 16, Game::drawy, 16, 32, 64, 344, 16, 32);

				g1 = GFX::Gamma(0, 0, 0, 0, 0, 0, 0, (200-(fintimer-600))*5);

				GFX::Push(Game::drawx+50/1+32, Game::drawy+16, 16, 16, 336, 16, 16, 16);
				GFX::PushGamma(Game::drawx + 60-24, Game::drawy - 10-24, 94, 94, 49, 457, 46, 46, g1);
			}
			else if (fintimer < 1000)
			{
				GFX::Push(Game::drawx+16+50/4, Game::drawy, 32, 32, 80, 48, 32, 32);
				GFX::Push(Game::drawx + 60 + 16, Game::drawy, 16, 32, 64, 344, 16, 32);

				GFX::Push(Game::drawx+50/1+32, Game::drawy+16, 16, 16, 336, 16, 16, 16);
			}
			else if (fintimer < 1200)
			{
				if (fintimer == 1100)
					Audio::Sound::Play(GameAudio::pick);
				g1 = GFX::Gamma(0, 255, 0, 255, 0, 255, 0, (105 + fintimer % 10 * 10) * (1000 - fintimer + 100) / 100.0);

				GFX::Push(Game::drawx + 60 + 16, Game::drawy, 16, 32, 64, 344, 16, 32);
				GFX::Push(Game::drawx+16+50/4, Game::drawy, 32, 32, 80, 48, 32, 32);

				if (fintimer < 1100)
					GFX::Push(Game::drawx+50/1+32, Game::drawy+16, 16, 16, 336, 16, 16, 16);
			}
			else if (fintimer < 1300)
			{
				if (fintimer == 1200)
					Audio::Sound::Play(GameAudio::tp1);

				g1 = GFX::Gamma(0, 255, 0, 255, 0, 255, 0, (105 + fintimer % 10 * 10) * (1300 - fintimer) / 100.0);

				GFX::PushGamma(Game::drawx + 60 + 16, Game::drawy, 16, 32, 64, 344, 16, 32, g1);
                GFX::Push(Game::drawx+16+50/4, Game::drawy, 32, 32, 80, 48, 32, 32);
			}
			else
			{
				if (fintimer > 1555) fintimer = 1555;

				g1 = GFX::Gamma(0, 255, 0, 255, 0, 255, 0, fintimer - 1300);

				GFX::Push(768/2-128, 64, 256, 128, 640, 128, 256, 128);

				GFX::Push(Game::drawx+16+50/4, Game::drawy, 32, 32, 80, 48, 32, 32);

			}

			fintimer++;
		}
	}
}