#define MAINARGS_DEFS
#define GFX_DEFS
#define AUDIO_DEFS
#define MSG_DEFS
#define SDL_DEFS
#define GL_DEFS
#define SYS_DEFS
#define LOOPS_ENTRYPOINT_DEFS
#define INPUT_DEFS
#define GAME_DEFS
#include "header.h"

#define TEXNAME "texture"

static int argc;
static char **argv;

static SDL_Window *winhandle;
static SDL_GLContext conhandle;

namespace ShaderTexts
{
	namespace x2shader
	{
		const char *vertex = 1+R"(
#version 430
in layout(location = 0) vec2 pos;
out vec2 vPos;
void main()
{
	vPos = pos;
	gl_Position = vec4(pos.x, pos.y, 0, 1);
})";
		const char *fragment = 1+R"(
#version 430
uniform layout(location = 0) sampler2D tex;
in vec2 vPos;
out vec4 color;
void main()
{
	color = texture(tex, (vPos + 1) / 2);
})";
	}

	namespace stdshader
	{
		const char *vertex = 1+R"(
#version 430
uniform layout(location = 3) vec2 fac;
in layout(location = 0) vec2 vpos;
in layout(location = 1) vec2 tpos;
in layout(location = 2) vec4 c_off;
in layout(location = 3) vec4 c_mult;
out vec2 var_tpos;
out vec4 var_c_off;
out vec4 var_c_mult;
void main()
{
	var_tpos = tpos;
	var_c_off = c_off;
	var_c_mult = c_mult;
	gl_Position = vec4(vpos.x * fac.x - 1, vpos.y * fac.y + 1, 0, 1);
})";
		const char *fragment = 1+R"(
#version 430
uniform layout(location = 0) sampler2D tex;
in vec2 var_tpos;
in vec4 var_c_off;
in vec4 var_c_mult;
out vec4 color;
void main()
{
	color = texture(tex, var_tpos / )" TEXSIZE_STRING R"();
	color.x *= var_c_mult.x;
	color.y *= var_c_mult.y;
	color.z *= var_c_mult.z;
	color.w *= var_c_mult.w;
	color   += var_c_off;
})";
	}
}

namespace Config
{


	namespace Volume
	{
		static float music = 1;

		void Music(float val)
		{
			if (val > 1) val = 1;
			else if (val < 0) val = 0;
			music = val;
		}

		void Sound(float val)
		{
			if (val > 1) val = 1;
			else if (val < 0) val = 0;
			Mix_Volume(-1, val * 127);
		}
	}
}

namespace GFX
{
	static GLuint fbo, rendertex, maintex, prog2x, progstd, vbofsquad, vaofsquad, vboqueue, vaoqueue;
	static unsigned int queuepos = 0;
	static const int win_xx = 768, win_yy = 512;
	static bool is_winzoom_float = 0;
	static float win_zoom_float;
	static int win_real_xx, win_real_yy, win_zoom;
	static const float clearR = 0, clearG = 0, clearB = 0, clearA = 1;
	const double Pi_d = std::atan(1)*4;
	const float Pi_f = std::atan(1)*4;

	const GammaData stdgamma = Gamma(0, 255, 0, 255, 0, 255, 0, 255);

	namespace Viewport
	{
		static short x = 0, y = 0, w = win_xx, h = win_yy;
	}

	static GLuint CreateShader(const char *vertex, const char *fragment)
	{
		static unsigned int idcounter;
		int id = idcounter++;

		GLuint ret, vsh, fsh;
		ret = glCreateProgram();
		if (!ret)
			Sys::ErrExit("glCreateProgram() failed.");

		vsh = glCreateShader(GL_VERTEX_SHADER);
		fsh = glCreateShader(GL_FRAGMENT_SHADER);
		if (!(vsh && fsh))
			Sys::ErrExit("glCreateShader() failed.");

		glAttachShader(ret, vsh);
		glAttachShader(ret, fsh);
		glShaderSource(vsh, 1, &vertex, 0);
		glShaderSource(fsh, 1, &fragment, 0);
		glCompileShader(vsh);
		glCompileShader(fsh);

		GLint vstat, fstat;
		glGetShaderiv(vsh, GL_COMPILE_STATUS, &vstat);
		glGetShaderiv(fsh, GL_COMPILE_STATUS, &fstat);
		bool vnok = vstat != GL_TRUE, fnok = fstat != GL_TRUE;
		if (vnok || fnok)
		{
			char *vlog, *flog;
			glGetShaderiv(vsh, GL_INFO_LOG_LENGTH, &vstat);
			glGetShaderiv(fsh, GL_INFO_LOG_LENGTH, &fstat);
			if (vstat)
			{
				vlog = new char[vstat];
				glGetShaderInfoLog(vsh, vstat, 0, vlog);
			}
			else
				vlog = (char *) "<empty>";
			if (fstat)
			{
				flog = new char[fstat];
				glGetShaderInfoLog(fsh, fstat, 0, flog);
			}
			else
				flog = (char *) "<empty>";

			Msg::MsgStream << Msg::Clear << "Shader program #" << id;
			if (vnok)
				Msg::MsgStream << "\nVertex shader compilation failed.";
			if (fnok)
				Msg::MsgStream << "\nFragment shader compilation failed.";
			Msg::MsgStream << "\n\n  Vertex log:\n" << vlog << "\n  Fragment log:\n" << flog << Sys::ErrExit;
		}
		glLinkProgram(ret);
		glGetProgramiv(ret, GL_LINK_STATUS, &vstat);
		if (vstat != GL_TRUE)
		{
			glGetProgramiv(ret, GL_INFO_LOG_LENGTH, &vstat);
			char *log;
			if (vstat)
			{
				log = new char[vstat];
				glGetProgramInfoLog(ret, vstat, 0, log);
			}
			else
				log = (char *) "<empty>";

			Msg::MsgStream << Msg::Clear << "Shader program #" << id << "\n Linking failed.\n\n  Log:\n" << log << Sys::ErrExit;
		}
		return ret;
	}

	static void Init()
	{
		SDL_GL_SetSwapInterval(0);

		glGenFramebuffers(1, &fbo);
		glGenTextures(1, &rendertex);
		glBindFramebuffer(GL_DRAW_FRAMEBUFFER, fbo);
		glActiveTexture(GL_TEXTURE0);
		glBindTexture(GL_TEXTURE_2D, rendertex);
		glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, win_xx, win_yy, 0, GL_RGBA, GL_UNSIGNED_BYTE, 0);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
		glFramebufferTexture(GL_DRAW_FRAMEBUFFER, GL_COLOR_ATTACHMENT0, rendertex, 0);
		if (glCheckFramebufferStatus(GL_DRAW_FRAMEBUFFER) != GL_FRAMEBUFFER_COMPLETE)
			Sys::ErrExit("Framebuffer is incomplete. Dafuq?");

		glEnable(GL_BLEND);
		glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

		FILE *infile = std::fopen(TEXNAME, "rb");
		auto Get = [&]() -> uint8_t
		{
			if (std::ferror(infile))
				Sys::ErrExit("Texture reading error!");
			return fgetc(infile);
		};

		if (std::ferror(infile))
			Sys::ErrExit("Can't load `texture`.");
		char buf;
		int tmp1, tmp2;
		if (buf = Get(), buf != 0)
			Sys::ErrExit("Texture: `0x00 != 0`!");
		if (buf = Get(), buf != 0)
			Sys::ErrExit("Texture: `0x01 != 0`!");
		if (buf = Get(), buf != 2)
			Sys::ErrExit("Texture: `0x02 != 2`!");
		if (buf = Get(), buf != 0)
			Sys::ErrExit("Texture: `0x03 != 0`!");
		if (buf = Get(), buf != 0)
			Sys::ErrExit("Texture: `0x04 != 0`!");
		if (buf = Get(), buf != 0)
			Sys::ErrExit("Texture: `0x05 != 0`!");
		if (buf = Get(), buf != 0)
			Sys::ErrExit("Texture: `0x06 != 0`!");
		if (buf = Get(), buf != 0)
			Sys::ErrExit("Texture: `0x07 != 0`!");
		if (buf = Get(), buf != 0)
			Sys::ErrExit("Texture: `0x08 != 0`!");
		if (buf = Get(), buf != 0)
			Sys::ErrExit("Texture: `0x09 != 0`!");
		if (buf = Get(), buf != 0)
			Sys::ErrExit("Texture: `0x0A != 0`!");
		if (buf = Get(), buf != 0)
			Sys::ErrExit("Texture: `0x0B != 0`!");
		buf = Get();
		tmp1 = buf;
		buf = Get();
		tmp1 = buf << 8 | tmp1;
		buf = Get();
		tmp2 = buf;
		buf = Get();
		tmp2 = buf << 8 | tmp2;
		if (buf = Get(), buf != 0x20)
			Sys::ErrExit("Texture: `0x10 != 32`!");
		buf = Get();
		if (tmp1 != tmp2 || tmp1 != TEXSIZE)
			Msg::MsgStream << Msg::Clear << "Texture must be " << TEXSIZE << 'x' << TEXSIZE << '!' << Sys::ErrExit;

		uint8_t *textemp = new uint8_t[TEXSIZE * TEXSIZE * 4];
		bool normx = buf & (1 << 3), normy = buf & (1 << 4);
		for (int i = 0; i < TEXSIZE; i++)
		{
			for (int j = 0; j < tmp2; j++)
			{
				uint8_t *ptr = textemp + ((normy ? i : TEXSIZE-1-i) * tmp1 + (normx ? j : TEXSIZE-1-j)) * 4;
				ptr[2] = Get();
				ptr[1] = Get();
				ptr[0] = Get();
				ptr[3] = Get();
			}
		}

		std::fclose(infile);
		glActiveTexture(GL_TEXTURE1);
		glGenTextures(1, &maintex);
		glBindTexture(GL_TEXTURE_2D, maintex);
		glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA8, tmp1, tmp1, 0, GL_RGBA, GL_UNSIGNED_BYTE, textemp);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
		delete [] textemp;

		prog2x = CreateShader(ShaderTexts::x2shader::vertex, ShaderTexts::x2shader::fragment);
		progstd = CreateShader(ShaderTexts::stdshader::vertex, ShaderTexts::stdshader::fragment);

		glProgramUniform1i(prog2x, 0, 0);
		glProgramUniform1i(progstd, 0, 1);
		glProgramUniform2f(progstd, 3, 2.0 / (win_xx), -2.0 / (win_yy));

		GLfloat fsquaddata[12] = {-1, -1, +1, -1, -1, +1, -1, +1, +1, -1, +1, +1};
		glGenBuffers(1, &vbofsquad);
		glBindBuffer(GL_ARRAY_BUFFER, vbofsquad);
		glBufferData(GL_ARRAY_BUFFER, sizeof fsquaddata, fsquaddata, GL_STATIC_DRAW);
		glGenVertexArrays(1, &vaofsquad);
		glBindVertexArray(vaofsquad);
		glEnableVertexAttribArray(0);
		glVertexAttribPointer(0, 2, GL_FLOAT, GL_FALSE, 0, (void *) 0);

		glGenBuffers(1, &vboqueue);
		glBindBuffer(GL_ARRAY_BUFFER, vboqueue);
		glBufferData(GL_ARRAY_BUFFER, IMAGEBUFFERSIZE * 48 * sizeof (float), 0, GL_DYNAMIC_DRAW);
		glGenVertexArrays(1, &vaoqueue);
		glBindVertexArray(vaoqueue);
		glEnableVertexAttribArray(0);
		glEnableVertexAttribArray(1);
		glEnableVertexAttribArray(2);
		glEnableVertexAttribArray(3);
		glVertexAttribPointer(0, 2, GL_FLOAT, GL_FALSE, sizeof (float) * 12, (void *) (sizeof (float) * 0));
		glVertexAttribPointer(1, 2, GL_FLOAT, GL_FALSE, sizeof (float) * 12, (void *) (sizeof (float) * 2));
		glVertexAttribPointer(2, 4, GL_FLOAT, GL_FALSE, sizeof (float) * 12, (void *) (sizeof (float) * 4));
		glVertexAttribPointer(3, 4, GL_FLOAT, GL_FALSE, sizeof (float) * 12, (void *) (sizeof (float) * 8));
	}

	GammaData Gamma(short rmin, short rmax, short gmin, short gmax, short bmin, short bmax, short amin, short amax) // rmin rmax gmin gmax bmin bmax amin amax
	{
		GammaData ret;
		ret.g[0] = rmin / 255.0;
		ret.g[1] = gmin / 255.0;
		ret.g[2] = bmin / 255.0;
		ret.g[3] = amin / 255.0;
		ret.g[4] = (rmax - rmin) / 255.0;
		ret.g[5] = (gmax - gmin) / 255.0;
		ret.g[6] = (bmax - bmin) / 255.0;
		ret.g[7] = (amax - amin) / 255.0;
		return ret;
	}

	void Push(float x, float y, float w, float h, float tx, float ty, float tw, float th)
	{
		if (queuepos == IMAGEBUFFERSIZE / 2)
			Sys::ErrExit("Dafuq? Render queue overflow!");
		float data[4][12] = {{x    , y    , tx   , ty   , 0,0,0,0,1,1,1,1},
							 {x+w  , y    , tx+tw, ty   , 0,0,0,0,1,1,1,1},
							 {x+w  , y+h  , tx+tw, ty+th, 0,0,0,0,1,1,1,1},
							 {x    , y+h  , tx   , ty+th, 0,0,0,0,1,1,1,1}};
		glBufferSubData(GL_ARRAY_BUFFER, sizeof (float) * 48 * queuepos, sizeof (float) * 48, data);
		queuepos++;
	}

	void PushExt(float x, float y, float w, float h, float tx, float ty, float tw, float th, float angle, float rx, float ry)
	{
		if (queuepos == IMAGEBUFFERSIZE / 2)
			Sys::ErrExit("Dafuq? Render queue overflow!");
		static float q = std::atan(1)*2;
		using std::sin;
		using std::cos;
		float xx = cos(angle), xy = sin(angle), yx = cos(angle+q), yy = sin(angle+q);
		float xo = x + rx, yo = y + ry;
		x = -rx;
		y = -ry;
		float data[4][12] = {{ x   *xx+ y   *yx+xo,  x   *xy+ y   *yy+yo, tx   , ty   , 0,0,0,0,1,1,1,1},
							 {(x+w)*xx+ y   *yx+xo, (x+w)*xy+ y   *yy+yo, tx+tw, ty   , 0,0,0,0,1,1,1,1},
							 {(x+w)*xx+(y+h)*yx+xo, (x+w)*xy+(y+h)*yy+yo, tx+tw, ty+th, 0,0,0,0,1,1,1,1},
							 { x   *xx+(y+h)*yx+xo,  x   *xy+(y+h)*yy+yo, tx   , ty+th, 0,0,0,0,1,1,1,1}};
		glBufferSubData(GL_ARRAY_BUFFER, sizeof (float) * 48 * queuepos, sizeof (float) * 48, data);
		queuepos++;
	}

	void PushGamma(float x, float y, float w, float h, float tx, float ty, float tw, float th, GammaData g)
	{
		if (queuepos == IMAGEBUFFERSIZE / 2)
			Sys::ErrExit("Dafuq? Render queue overflow!");
		float data[4][12] = {{x    , y    , tx   , ty   , g.g[0],g.g[1],g.g[2],g.g[3],g.g[4],g.g[5],g.g[6],g.g[7]},
							 {x+w  , y    , tx+tw, ty   , g.g[0],g.g[1],g.g[2],g.g[3],g.g[4],g.g[5],g.g[6],g.g[7]},
							 {x+w  , y+h  , tx+tw, ty+th, g.g[0],g.g[1],g.g[2],g.g[3],g.g[4],g.g[5],g.g[6],g.g[7]},
							 {x    , y+h  , tx   , ty+th, g.g[0],g.g[1],g.g[2],g.g[3],g.g[4],g.g[5],g.g[6],g.g[7]}};
		glBufferSubData(GL_ARRAY_BUFFER, sizeof (float) * 48 * queuepos, sizeof (float) * 48, data);
		queuepos++;
	}

	void PushSeparateGamma(float x, float y, float w, float h, float tx, float ty, float tw, float th, GammaData g1, GammaData g2, GammaData g3, GammaData g4)
	{
		if (queuepos == IMAGEBUFFERSIZE / 2)
			Sys::ErrExit("Dafuq? Render queue overflow!");
		float data[4][12] = {{x    , y    , tx   , ty   , g1.g[0],g1.g[1],g1.g[2],g1.g[3],g1.g[4],g1.g[5],g1.g[6],g1.g[7]},
							 {x+w  , y    , tx+tw, ty   , g2.g[0],g2.g[1],g2.g[2],g2.g[3],g2.g[4],g2.g[5],g2.g[6],g2.g[7]},
							 {x+w  , y+h  , tx+tw, ty+th, g4.g[0],g4.g[1],g4.g[2],g4.g[3],g4.g[4],g4.g[5],g4.g[6],g4.g[7]},
							 {x    , y+h  , tx   , ty+th, g3.g[0],g3.g[1],g3.g[2],g3.g[3],g3.g[4],g3.g[5],g3.g[6],g3.g[7]}};
		glBufferSubData(GL_ARRAY_BUFFER, sizeof (float) * 48 * queuepos, sizeof (float) * 48, data);
		queuepos++;
	}

	void PushExtGamma(float x, float y, float w, float h, float tx, float ty, float tw, float th, float angle, float rx, float ry, GammaData g)
	{
		if (queuepos == IMAGEBUFFERSIZE / 2)
			Sys::ErrExit("Dafuq? Render queue overflow!");
		static float q = std::atan(1)*2;
		using std::sin;
		using std::cos;
		float xx = cos(angle), xy = sin(angle), yx = cos(angle+q), yy = sin(angle+q);
		float xo = x + rx, yo = y + ry;
		x = -rx;
		y = -ry;
		float data[4][12] = {{ x   *xx+ y   *yx+xo,  x   *xy+ y   *yy+yo, tx   , ty   , g.g[0],g.g[1],g.g[2],g.g[3],g.g[4],g.g[5],g.g[6],g.g[7]},
							 {(x+w)*xx+ y   *yx+xo, (x+w)*xy+ y   *yy+yo, tx+tw, ty   , g.g[0],g.g[1],g.g[2],g.g[3],g.g[4],g.g[5],g.g[6],g.g[7]},
							 {(x+w)*xx+(y+h)*yx+xo, (x+w)*xy+(y+h)*yy+yo, tx+tw, ty+th, g.g[0],g.g[1],g.g[2],g.g[3],g.g[4],g.g[5],g.g[6],g.g[7]},
							 { x   *xx+(y+h)*yx+xo,  x   *xy+(y+h)*yy+yo, tx   , ty+th, g.g[0],g.g[1],g.g[2],g.g[3],g.g[4],g.g[5],g.g[6],g.g[7]}};
		glBufferSubData(GL_ARRAY_BUFFER, sizeof (float) * 48 * queuepos, sizeof (float) * 48, data);
		queuepos++;
	}

	void PushExtSeparateGamma(float x, float y, float w, float h, float tx, float ty, float tw, float th, float angle, float rx, float ry,
							  GammaData g1, GammaData g2, GammaData g3, GammaData g4)
	{
		if (queuepos == IMAGEBUFFERSIZE / 2)
			Sys::ErrExit("Dafuq? Render queue overflow!");
		static float q = std::atan(1)*2;
		using std::sin;
		using std::cos;
		float xx = cos(angle), xy = sin(angle), yx = cos(angle+q), yy = sin(angle+q);
		float xo = x + rx, yo = y + ry;
		x = -rx;
		y = -ry;
		float data[4][12] = {{ x   *xx+ y   *yx+xo,  x   *xy+ y   *yy+yo, tx   , ty   , g1.g[0],g1.g[1],g1.g[2],g1.g[3],g1.g[4],g1.g[5],g1.g[6],g1.g[7]},
							 {(x+w)*xx+ y   *yx+xo, (x+w)*xy+ y   *yy+yo, tx+tw, ty   , g2.g[0],g2.g[1],g2.g[2],g2.g[3],g2.g[4],g2.g[5],g2.g[6],g2.g[7]},
							 {(x+w)*xx+(y+h)*yx+xo, (x+w)*xy+(y+h)*yy+yo, tx+tw, ty+th, g4.g[0],g4.g[1],g4.g[2],g4.g[3],g4.g[4],g4.g[5],g4.g[6],g4.g[7]},
							 { x   *xx+(y+h)*yx+xo,  x   *xy+(y+h)*yy+yo, tx   , ty+th, g3.g[0],g3.g[1],g3.g[2],g3.g[3],g3.g[4],g3.g[5],g3.g[6],g3.g[7]}};
		glBufferSubData(GL_ARRAY_BUFFER, sizeof (float) * 48 * queuepos, sizeof (float) * 48, data);
		queuepos++;
	}



	void PosPush(unsigned int pos, float x, float y, float w, float h, float tx, float ty, float tw, float th)
	{
		float data[4][12] = {{x    , y    , tx   , ty   , 0,0,0,0,1,1,1,1},
							 {x+w  , y    , tx+tw, ty   , 0,0,0,0,1,1,1,1},
							 {x+w  , y+h  , tx+tw, ty+th, 0,0,0,0,1,1,1,1},
							 {x    , y+h  , tx   , ty+th, 0,0,0,0,1,1,1,1}};
		glBufferSubData(GL_ARRAY_BUFFER, sizeof (float) * 48 * (IMAGEBUFFERSIZE/2+pos), sizeof (float) * 48, data);
	}

	void PosPushExt(unsigned int pos, float x, float y, float w, float h, float tx, float ty, float tw, float th, float angle, float rx, float ry)
	{
		static float q = std::atan(1)*2;
		using std::sin;
		using std::cos;
		float xx = cos(angle), xy = sin(angle), yx = cos(angle+q), yy = sin(angle+q);
		float xo = x + rx, yo = y + ry;
		x = -rx;
		y = -ry;
		float data[4][12] = {{ x   *xx+ y   *yx+xo,  x   *xy+ y   *yy+yo, tx   , ty   , 0,0,0,0,1,1,1,1},
							 {(x+w)*xx+ y   *yx+xo, (x+w)*xy+ y   *yy+yo, tx+tw, ty   , 0,0,0,0,1,1,1,1},
							 {(x+w)*xx+(y+h)*yx+xo, (x+w)*xy+(y+h)*yy+yo, tx+tw, ty+th, 0,0,0,0,1,1,1,1},
							 { x   *xx+(y+h)*yx+xo,  x   *xy+(y+h)*yy+yo, tx   , ty+th, 0,0,0,0,1,1,1,1}};
		glBufferSubData(GL_ARRAY_BUFFER, sizeof (float) * 48 * (IMAGEBUFFERSIZE/2+pos), sizeof (float) * 48, data);
	}

	void PosPushGamma(unsigned int pos, float x, float y, float w, float h, float tx, float ty, float tw, float th, GammaData g)
	{
		float data[4][12] = {{x    , y    , tx   , ty   , g.g[0],g.g[1],g.g[2],g.g[3],g.g[4],g.g[5],g.g[6],g.g[7]},
							 {x+w  , y    , tx+tw, ty   , g.g[0],g.g[1],g.g[2],g.g[3],g.g[4],g.g[5],g.g[6],g.g[7]},
							 {x+w  , y+h  , tx+tw, ty+th, g.g[0],g.g[1],g.g[2],g.g[3],g.g[4],g.g[5],g.g[6],g.g[7]},
							 {x    , y+h  , tx   , ty+th, g.g[0],g.g[1],g.g[2],g.g[3],g.g[4],g.g[5],g.g[6],g.g[7]}};
		glBufferSubData(GL_ARRAY_BUFFER, sizeof (float) * 48 * (IMAGEBUFFERSIZE/2+pos), sizeof (float) * 48, data);
	}

	void PosPushSeparateGamma(unsigned int pos, float x, float y, float w, float h, float tx, float ty, float tw, float th,
							  GammaData g1, GammaData g2, GammaData g3, GammaData g4)
	{
		float data[4][12] = {{x    , y    , tx   , ty   , g1.g[0],g1.g[1],g1.g[2],g1.g[3],g1.g[4],g1.g[5],g1.g[6],g1.g[7]},
							 {x+w  , y    , tx+tw, ty   , g2.g[0],g2.g[1],g2.g[2],g2.g[3],g2.g[4],g2.g[5],g2.g[6],g2.g[7]},
							 {x+w  , y+h  , tx+tw, ty+th, g4.g[0],g4.g[1],g4.g[2],g4.g[3],g4.g[4],g4.g[5],g4.g[6],g4.g[7]},
							 {x    , y+h  , tx   , ty+th, g3.g[0],g3.g[1],g3.g[2],g3.g[3],g3.g[4],g3.g[5],g3.g[6],g3.g[7]}};
		glBufferSubData(GL_ARRAY_BUFFER, sizeof (float) * 48 * (IMAGEBUFFERSIZE/2+pos), sizeof (float) * 48, data);
	}

	void PosPushExtGamma(unsigned int pos, float x, float y, float w, float h, float tx, float ty, float tw, float th, float angle, float rx, float ry,
						 GammaData g)
	{
		static float q = std::atan(1)*2;
		using std::sin;
		using std::cos;
		float xx = cos(angle), xy = sin(angle), yx = cos(angle+q), yy = sin(angle+q);
		float xo = x + rx, yo = y + ry;
		x = -rx;
		y = -ry;
		float data[4][12] = {{ x   *xx+ y   *yx+xo,  x   *xy+ y   *yy+yo, tx   , ty   , g.g[0],g.g[1],g.g[2],g.g[3],g.g[4],g.g[5],g.g[6],g.g[7]},
							 {(x+w)*xx+ y   *yx+xo, (x+w)*xy+ y   *yy+yo, tx+tw, ty   , g.g[0],g.g[1],g.g[2],g.g[3],g.g[4],g.g[5],g.g[6],g.g[7]},
							 {(x+w)*xx+(y+h)*yx+xo, (x+w)*xy+(y+h)*yy+yo, tx+tw, ty+th, g.g[0],g.g[1],g.g[2],g.g[3],g.g[4],g.g[5],g.g[6],g.g[7]},
							 { x   *xx+(y+h)*yx+xo,  x   *xy+(y+h)*yy+yo, tx   , ty+th, g.g[0],g.g[1],g.g[2],g.g[3],g.g[4],g.g[5],g.g[6],g.g[7]}};
		glBufferSubData(GL_ARRAY_BUFFER, sizeof (float) * 48 * (IMAGEBUFFERSIZE/2+pos), sizeof (float) * 48, data);
	}

	void PosPushExtSeparateGamma(unsigned int pos, float x, float y, float w, float h, float tx, float ty, float tw, float th, float angle, float rx, float ry,
								 GammaData g1, GammaData g2, GammaData g3, GammaData g4)
	{
		static float q = std::atan(1)*2;
		using std::sin;
		using std::cos;
		float xx = cos(angle), xy = sin(angle), yx = cos(angle+q), yy = sin(angle+q);
		float xo = x + rx, yo = y + ry;
		x = -rx;
		y = -ry;
		float data[4][12] = {{ x   *xx+ y   *yx+xo,  x   *xy+ y   *yy+yo, tx   , ty   , g1.g[0],g1.g[1],g1.g[2],g1.g[3],g1.g[4],g1.g[5],g1.g[6],g1.g[7]},
							 {(x+w)*xx+ y   *yx+xo, (x+w)*xy+ y   *yy+yo, tx+tw, ty   , g2.g[0],g2.g[1],g2.g[2],g2.g[3],g2.g[4],g2.g[5],g2.g[6],g2.g[7]},
							 {(x+w)*xx+(y+h)*yx+xo, (x+w)*xy+(y+h)*yy+yo, tx+tw, ty+th, g4.g[0],g4.g[1],g4.g[2],g4.g[3],g4.g[4],g4.g[5],g4.g[6],g4.g[7]},
							 { x   *xx+(y+h)*yx+xo,  x   *xy+(y+h)*yy+yo, tx   , ty+th, g3.g[0],g3.g[1],g3.g[2],g3.g[3],g3.g[4],g3.g[5],g3.g[6],g3.g[7]}};
		glBufferSubData(GL_ARRAY_BUFFER, sizeof (float) * 48 * (IMAGEBUFFERSIZE/2+pos), sizeof (float) * 48, data);
	}


	static void DrawQueue()
	{
		glDrawArrays(GL_QUADS, 0, queuepos * 4);
		queuepos = 0;
	}

	void DrawPos(unsigned int pos, unsigned int len)
	{
		glDrawArrays(GL_QUADS, (IMAGEBUFFERSIZE + pos) * 4, len * 4);
	}

	short WinXX()
	{
		return win_xx;
	}

	short WinYY()
	{
		return win_yy;
	}

	static void Resize(int w, int h, bool isfloat)
	{
		win_real_xx = w;
		win_real_yy = h;

		is_winzoom_float = isfloat;
		if (isfloat)
		{
			glActiveTexture(GL_TEXTURE0);
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
			glActiveTexture(GL_TEXTURE1);
			float ratio = (float)w / (float)h;
			float targetratio = (float)win_xx / (float)win_yy;

			if (ratio > targetratio)
			{
				Viewport::y = 0;
				Viewport::h = h;

				float realxx = h * targetratio;
				Viewport::x = (w - realxx) / 2;
				Viewport::w = realxx;

				win_zoom_float = (float)h / (float)win_yy;
			}
			else
			{
				Viewport::x = 0;
				Viewport::w = w;

				float realyy = w / targetratio;
				Viewport::y = (h - realyy) / 2;
				Viewport::h = realyy;

				win_zoom_float = (float)w / (float)win_xx;
			}
		}
		else
		{
			glActiveTexture(GL_TEXTURE0);
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
			glActiveTexture(GL_TEXTURE1);
			int wfac, hfac;
			wfac = w / win_xx;
			hfac = h / win_yy;
			win_zoom = wfac > hfac ? hfac : wfac;

			Viewport::w = win_xx * win_zoom;
			Viewport::h = win_yy * win_zoom;
			Viewport::x = (w - Viewport::w) / 2;
			Viewport::y = (h - Viewport::h) / 2;
		}
	}

	static void DrawFullscreenQuad()
	{
		SDL_GL_SwapWindow(winhandle);
		glUseProgram(prog2x);
		glBindFramebuffer(GL_DRAW_FRAMEBUFFER, 0);
		glViewport(0, 0, win_real_xx, win_real_yy);
		glClear(GL_COLOR_BUFFER_BIT);
		glViewport(Viewport::x, Viewport::y, Viewport::w, Viewport::h);
		glBindVertexArray(vaofsquad);
		glDrawArrays(GL_TRIANGLES, 0, 12);
		glUseProgram(progstd);
		glBindFramebuffer(GL_DRAW_FRAMEBUFFER, fbo);
		glViewport(0, 0, win_xx, win_yy);
		glBindVertexArray(vaoqueue);
		static float color[4] = {clearR, clearG, clearB, clearA};
		glClearBufferfv(GL_COLOR, 0, color);
	}
}

namespace Audio
{
	namespace Music
	{
		static Mix_Music *current = 0, *target = 0;
		static int time = 0;

		Type Load(const char *name)
		{
			Type ret = Mix_LoadMUS(name);
			if (!ret)
				Msg::MsgStream << Msg::Clear << "Can't load music file `" << name << "`." << Sys::ErrExit;
			return ret;
		}

		void Set(Type ptr)
		{
			target = ptr;
		}

		void Pause(bool val)
		{
			if (val)
				Mix_PauseMusic();
			else
				Mix_ResumeMusic();
		}
	}

	namespace Sound
	{
		Type Load(const char *name)
		{
			Type ret = Mix_LoadWAV(name);
			if (!ret)
				Msg::MsgStream << Msg::Clear << "Can't load sound file `" << name << "`." << Sys::ErrExit;
			return ret;
		}

		void Play(Type sound)
		{
			int ch = Mix_PlayChannelTimed(-1, sound, 0, -1);
			if (ch == -1) return;
			Mix_SetPanning(ch, 255, 255);
		}

		void Play(Type sound, float x, float y)
		{
			int ch = Mix_PlayChannelTimed(-1, sound, 0, -1);
			if (ch == -1) return;
			x *= AUDIOPOSMULTIPLICATOR;
			y *= AUDIOPOSMULTIPLICATOR;
			Mix_SetPanning(ch, 255.0 / (1 + (x+AUDIOHALFDISTBETWEENEARS)*(x+AUDIOHALFDISTBETWEENEARS) + y*y),
							   255.0 / (1 + (x-AUDIOHALFDISTBETWEENEARS)*(x-AUDIOHALFDISTBETWEENEARS) + y*y));
		}
	}

	static void Tick()
	{
		if (Music::current == Music::target)
		{
			if (Music::current == 0)
				Music::time = 0;
			else if (Music::time > 500)
				Music::time = 500;
			else if (Music::time < 500)
				Music::time += Sys::Delta();
		}
		else
		{
			if (Music::time < 0)
			{
				Music::time = 0;
				Music::current = Music::target;
				if (Music::current)
					Mix_PlayMusic(Music::target, -1);
				else
					Mix_HaltMusic();
			}
			else
				Music::time -= Sys::Delta();
		}
		float music = Config::Volume::music * Music::time * 0.0127 * 20;
		Mix_VolumeMusic(music <= 127 ? music : 127);
	}
}

namespace Input
{
	namespace Mouse
	{
		static int x, y, tx, ty;
		static uint32_t mask, prevmask;
		static bool dirs[4]; // up down left right
		static bool ins;

		void Show(bool n)
		{
			SDL_ShowCursor(n);
		}

		void Grab(bool n)
		{
			SDL_SetWindowGrab(winhandle, n ? SDL_TRUE : SDL_FALSE);
		}

		bool Inside()
		{
			return ins;
		}

		int PosX()
		{
			return tx;
		}

		int PosY()
		{
			return ty;
		}

		int AbsPosX()
		{
			return x;
		}

		int AbsPosY()
		{
			return y;
		}

		bool Up(unsigned char n)       {return !(mask & SDL_BUTTON(n) || prevmask & SDL_BUTTON(n));}
		bool Down(unsigned char n)     {return mask & SDL_BUTTON(n);}
		bool Hold(unsigned char n)     {return mask & SDL_BUTTON(n) && prevmask & SDL_BUTTON(n);}
		bool Pressed(unsigned char n)  {return mask & SDL_BUTTON(n) && !(prevmask & SDL_BUTTON(n));}
		bool Released(unsigned char n) {return !(mask & SDL_BUTTON(n)) && prevmask & SDL_BUTTON(n);}

		bool WheelUp()    {return dirs[0];}
		bool WheelDown()  {return dirs[1];}
		bool WheelLeft()  {return dirs[2];}
		bool WheelRight() {return dirs[3];}

		unsigned char PressedNow() {for (int i = 1; i < 32; i++) if (mask & SDL_BUTTON(i) && !(prevmask & SDL_BUTTON(i))) return i; return 0;}

		bool RectHover(int x, int y, int w, int h)
		{
			return PosX() >= x && PosY() >= y && PosX() < x + w && PosY() < y + h;
		}

		bool RectClick(int x, int y, int w, int h)
		{
			return Pressed(1) && PosX() >= x && PosY() >= y && PosX() < x + w && PosY() < y + h;
		}

		bool RectHold(int x, int y, int w, int h)
		{
			return Down(1) && PosX() >= x && PosY() >= y && PosX() < x + w && PosY() < y + h;
		}

		namespace Control
		{
			static void Tick()
			{
				ins = SDL_GetMouseFocus();
				dirs[0] = dirs[1] = dirs[2] = dirs[3] = 0;
				prevmask = mask;
				mask = SDL_GetMouseState(&x, &y);
				if (GFX::is_winzoom_float)
				{
					tx = (x - GFX::Viewport::x) / GFX::win_zoom_float;
					ty = (y - GFX::Viewport::y) / GFX::win_zoom_float;
				}
				else
				{
					tx = (x - GFX::Viewport::x) / GFX::win_zoom;
					ty = (y - GFX::Viewport::y) / GFX::win_zoom;
				}
			}
		}
	}

	namespace Keyboard
	{
		static int len;
		static union
		{
			struct
			{
				unsigned char a : 4, b : 4;
			}
			parts;
			unsigned char sum;
		}
		*board;
		static unsigned int last;

		unsigned int NowPressed()
		{
			return last;
		}

		bool Up(unsigned int key)
		{
			return board[key].parts.a == 0;
		}

		bool Down(unsigned int key)
		{
			return board[key].parts.a == 2 || board[key].parts.a == 1;
		}

		bool Hold(unsigned int key)
		{
			return board[key].parts.a == 2;
		}

		bool Pressed(unsigned int key)
		{
			return board[key].parts.a == 1;
		}

		bool Released(unsigned int key)
		{
			return board[key].parts.a == 3;
		}

		bool Repeated(unsigned int key)
		{
			return board[key].parts.b == 1;
		}

		bool PressedOrRepeated(unsigned int key)
		{
			return board[key].parts.b == 1 || board[key].parts.a == 1;
		}

		namespace Control
		{
			static void Tick()
			{
				last = 0;
				for (int i = 0; i < len; i++)
				{
					if (board[i].sum)
					{
						board[i].parts.b = 0;
						if (board[i].parts.a == 1)
							board[i].parts.a = 2;
						else if (board[i].parts.a == 3)
							board[i].parts.a = 0;
					}
				}
			}

			static void Press(unsigned int key)
			{
				board[key].parts.a = 1;
				last = key;
			}

			static void Release(unsigned int key)
			{
				board[key].parts.a = 3;
			}

			static void PressRepeat(unsigned int key)
			{
				board[key].parts.b = 1;
			}
		}
	}
}

namespace Sys
{
	static void (*atexitlist[ATEXITSTACKSIZE])();
	static unsigned int atexitlistpos = 0;
	static unsigned int delta;

	void Exit()
	{
		while (atexitlistpos > 0)
			atexitlist[--atexitlistpos]();
		std::exit(0);
		Inline::StopExecution();
	}

	void ErrExit(const char *txt)
	{
		SDL_ShowSimpleMessageBox(SDL_MESSAGEBOX_ERROR, APPNAME" - Error", txt, 0);
		Exit();
	}

	void AtExit(void (*ptr)())
	{
		if (atexitlistpos >= ATEXITSTACKSIZE)
			ErrExit("AtExit() stack overflow.");
		atexitlist[atexitlistpos++] = ptr;
	}

	unsigned int Delta()
	{
		return delta;
	}

	void Tick()
	{
		Audio::Tick();

		static bool fullscreen = 0;
		if (Input::Keyboard::Pressed(40) && (Input::Keyboard::Down(226) || Input::Keyboard::Down(230))) // alt+enter
			SDL_SetWindowFullscreen(winhandle, SDL_WINDOW_FULLSCREEN_DESKTOP * (fullscreen = !fullscreen));

		static int xx, yy;
		static bool floatzoom = 0;
		if (Input::Keyboard::Pressed(40) && (Input::Keyboard::Down(224) || Input::Keyboard::Down(228))) // ctrl+enter
		{
			floatzoom = !floatzoom;
			xx = 0;
		}

		static std::string glerr;
		static bool isglerr;
		while (GLenum err = glGetError())
		{
			isglerr = 1;
			switch (err)
			{
				case GL_INVALID_ENUM:                  glerr += "GL: Invalid enum.\n"; break;
				case GL_INVALID_VALUE:                 glerr += "GL: Invalid value.\n"; break;
				case GL_INVALID_OPERATION:             glerr += "GL: Invalid operation.\n"; break;
				case GL_INVALID_FRAMEBUFFER_OPERATION: glerr += "GL: Invalid framebuffer operation.\n"; break;
				case GL_OUT_OF_MEMORY:                 glerr += "GL: Out of memory.\n"; break;
				case GL_STACK_UNDERFLOW:               glerr += "GL: Stack underflow.\n"; break;
				case GL_STACK_OVERFLOW:                glerr += "GL: Stack overflow.\n"; break;
				default:                               glerr += "GL: Unknown error.\n"; break;
			}
		}
		if (isglerr)
			Sys::ErrExit(glerr.c_str());

		GFX::DrawQueue();

		static std::clock_t time1 = Inline::Time(), time2, time3;

		time2 = Inline::Time();

		if (time2 < time1) time1 = time2;

		if (time2 - time1 < TICKLEN)
		{
			SDL_Delay(TICKLEN - (time2 - time1));
		}

		time3 = time1;
		time1 = Inline::Time();
		delta = time1 - time3;

		GFX::DrawFullscreenQuad();

		int w, h;
		SDL_GetWindowSize(winhandle, &w, &h);
		if (w != xx || h != yy)
			GFX::Resize(xx = w, yy = h, floatzoom);

		Input::Keyboard::Control::Tick();

		static SDL_Event curevent;
		while (SDL_PollEvent(&curevent))
		{
			switch (curevent.type)
			{
			case SDL_QUIT:
				Exit();
				break;

			case SDL_KEYDOWN:
				if (curevent.key.repeat == 0)
				{
					Input::Keyboard::Control::Press(curevent.key.keysym.scancode);
				}
				else
				{
					Input::Keyboard::Control::PressRepeat(curevent.key.keysym.scancode);
				}
				break;

			case SDL_KEYUP:
				if (curevent.key.repeat == 0)
					Input::Keyboard::Control::Release(curevent.key.keysym.scancode);
				break;

			case SDL_MOUSEWHEEL:
				if (curevent.wheel.x != 0)
				{
					if (curevent.wheel.x > 0)
						Input::Mouse::dirs[3] = 1;
					else
						Input::Mouse::dirs[2] = 1;
				}
				if (curevent.wheel.y != 0)
				{
					if (curevent.wheel.y > 0)
						Input::Mouse::dirs[0] = 1;
					else
						Input::Mouse::dirs[1] = 1;
				}
			}
		}

		Input::Mouse::Control::Tick();
	}
}

namespace MainArgs
{
	const char *ExecutableName()
	{
		if (argc < 1)
			Sys::ErrExit("Can't get executable name: argc == 0.");
		return *argv;
	}

	unsigned int ArgCount()
	{
		return argc > 0 ? argc - 1 : 0;
	}

	const char *ArgAt(unsigned int n)
	{
		if (n >= ArgCount())
			Sys::ErrExit("Main() argument index out of range.");
		return argv[n+1];
	}

	static unsigned int lastarg = 0;

	bool ArgExist(const char *txt)
	{
		for (int i = 1; i < argc; i++)
		{
			int j = 0;
			while (1)
			{
				if (argv[i][j] != txt[j])
					goto br;
				if (!txt[j])
					break;
				j++;
			}
			lastarg = i - 1;
			return 1;
			br:;
		}
		return 0;
	}

	bool ArgWithPrefixExist(const char *txt)
	{
		for (int i = 1; i < argc; i++)
		{
			int j = 0;
			while (1)
			{
				if (!txt[j])
					break;
				if (argv[i][j] != txt[j])
					goto br;
				j++;
			}
			lastarg = i - 1;
			return 1;
		    br: ;
		}
		return 0;
	}

	unsigned int LastArg()
	{
		return lastarg;
	}
}

static void Init()
{
	auto SignalHandler = [](int sig) -> void
	{
		switch (sig)
		{
		case SIGINT: case SIGTERM: case SIGBREAK:
			Sys::Exit();
			break;

		case SIGILL:
			Sys::ErrExit("SIGILL: Illegal instruction.");
			break;

		case SIGFPE:
			Sys::ErrExit("SIGFPE: Floating point error.");
			break;

		case SIGSEGV:
			Sys::ErrExit("SIGSEGV: Segmentation violation.");
			break;

		case SIGABRT:
			Sys::ErrExit("SIGABRT: Abnormal termination.");
			break;

		default:
			Sys::ErrExit("Unknown signal.");
			break;
		}
	};

	int sigcounter = 0;
	sigcounter += SIG_ERR != std::signal(SIGINT  , SignalHandler);
	sigcounter += SIG_ERR != std::signal(SIGTERM , SignalHandler);
	sigcounter += SIG_ERR != std::signal(SIGBREAK, SignalHandler);
	sigcounter += SIG_ERR != std::signal(SIGILL  , SignalHandler);
	sigcounter += SIG_ERR != std::signal(SIGFPE  , SignalHandler);
	sigcounter += SIG_ERR != std::signal(SIGSEGV , SignalHandler);
	sigcounter += SIG_ERR != std::signal(SIGABRT , SignalHandler);
	if (sigcounter < 7)
		Msg::Warn("Warning: Signal handlers setup failed.");

	if (CLOCKS_PER_SEC < 1000)
		Sys::ErrExit("System clock resolution is too low.");

	std::set_terminate([](){Sys::ErrExit("Abnormal termination. (Probably out of memory.)");});
	std::set_unexpected([](){Sys::ErrExit("Unexpected exception.");});

	glewExperimental = 1;

	if (SDL_Init(SDL_INIT_VIDEO | SDL_INIT_EVENTS | SDL_INIT_NOPARACHUTE)) // Only noobs use parachutes! :D
		Sys::ErrExit("SDL init failed.");
	Sys::AtExit([](){SDL_Quit();});

	constexpr uint32_t winflags = SDL_WINDOW_OPENGL | SDL_WINDOW_HIDDEN | SDL_WINDOW_RESIZABLE;
	if (!(winhandle = SDL_CreateWindow(APPNAME, SDL_WINDOWPOS_UNDEFINED, SDL_WINDOWPOS_UNDEFINED, GFX::win_xx, GFX::win_yy, winflags)))
		Sys::ErrExit("Window init failed.");
	Sys::AtExit([](){SDL_DestroyWindow(winhandle);});
	SDL_SetWindowMinimumSize(winhandle, GFX::win_xx, GFX::win_yy);

	SDL_GL_SetAttribute(SDL_GL_CONTEXT_MAJOR_VERSION, 4);
	SDL_GL_SetAttribute(SDL_GL_CONTEXT_MINOR_VERSION, 3);
	SDL_GL_SetAttribute(SDL_GL_CONTEXT_PROFILE_MASK,  SDL_GL_CONTEXT_PROFILE_COMPATIBILITY);

	if (!(conhandle = SDL_GL_CreateContext(winhandle)))
		Sys::ErrExit("OpenGL 4.3 (compatibility profile) context init failed: your videodriver is too old or your videocard is too crappy.");
	Sys::AtExit([](){SDL_GL_DeleteContext(conhandle);});

	if (glewInit() != GLEW_OK)
		Sys::ErrExit("GLEW init failed.");

	if (Mix_Init(MIX_INIT_MP3 | MIX_INIT_OGG) != (MIX_INIT_MP3 | MIX_INIT_OGG))
		Sys::ErrExit("SDL mixer plugin init failed.");
	Sys::AtExit([](){Mix_Quit();});

	if (Mix_OpenAudio(22050 * (HQAUDIO + 1), AUDIO_S16LSB, 2, 2048))
		Sys::ErrExit("Audio channel opening failed.");
	Sys::AtExit([](){Mix_CloseAudio();});

	if ((sigcounter = Mix_AllocateChannels(AUDIOCHANNELS)) != AUDIOCHANNELS)
		Msg::MsgStream << Msg::Clear << "Warning: can't allocate " << AUDIOCHANNELS << " audio channels. " << sigcounter << " allocated." << Msg::Warn;

	SDL_GetKeyboardState(&Input::Keyboard::len);
	Input::Keyboard::board = new std::remove_pointer<decltype(Input::Keyboard::board)>::type[Input::Keyboard::len];
	Sys::AtExit([](){delete [] Input::Keyboard::board;});
	for (int i = 0; i < Input::Keyboard::len; i++)
		Input::Keyboard::board[i].sum = 0;

	GFX::Init();
	Game::Init();

	SDL_ShowWindow(winhandle);
	SDL_MaximizeWindow(winhandle);
	Input::Mouse::Show(0);
}

int main(int c, char **v)
{
	argc = c;
	argv = v;
	Init();

	Loops::EntryPoint();
	Sys::Exit();

	return 0;
}